struct sRect
{
	int left, top, right, bottom;

	BOOL IsPtInside(int x, int y) { return x >= left && y >= top && x < right && y < bottom; }
};


///////////////////////////////////////////////////////////////////////////////

// base class for our HUD elements
class cHudElement
{
public:
	BOOL m_bActive;
	sRect m_rect;

public:
	cHudElement() { m_bActive=FALSE; m_rect.left=m_rect.top=m_rect.right=m_rect.bottom=0; }
	virtual ~cHudElement() {}

	void Toggle() { m_bActive = !m_bActive; OnShow(m_bActive); }
	void Show() { if (!m_bActive) { m_bActive = TRUE; OnShow(TRUE); } }
	void Hide() { if (m_bActive) { m_bActive = FALSE; OnShow(FALSE); } }

	virtual void CalcPlacement() {}

	virtual void OnShow(BOOL bShow) {}

	virtual BOOL MouseClick(int x, int y) { return FALSE; }
	virtual BOOL MouseDblClick(int x, int y) { return FALSE; }
	virtual BOOL MouseDragDrop(int x, int y, BOOL start_drag, int cursor_mode) { return FALSE; }
	virtual BOOL OnKey(int ch, int keycode) { return FALSE; }

	virtual void Draw() = 0;
};

// a custom HUD element
class cHudElement_Something : public cHudElement
{
	int m_bgImage;

public:
	cHudElement_Something()
	{
		m_bgImage = ShockOverlay.GetBitmap("cyber", "iface\\");

		CalcPlacement();
	}

	virtual ~cHudElement_Something() {}

	virtual void CalcPlacement()
	{
		// TODO: calc suitable position based on Engine.GetCanvasSize and/or other overly elemens ShockOverlay.GetOverlayRect
		m_rect.left = 0;
		m_rect.top =  0;
		ShockOverlay.GetBitmapSize(m_bgImage, m_rect.right, m_rect.bottom);
		m_rect.right += m_rect.left;
		m_rect.bottom += m_rect.top;
	}

	virtual void Draw()
	{
		ShockOverlay.DrawBitmap(m_bgImage, m_rect.left, m_rect.top);
	}
};

// another custom HUD element
class cHudElement_SomethingElse : public cHudElement
{
	int m_bgImage;

public:
	cHudElement_SomethingElse()
	{
		m_bgImage = ShockOverlay.GetBitmap("ammodiag", "iface\\");

		CalcPlacement();
	}

	virtual ~cHudElement_SomethingElse() {}

	virtual void CalcPlacement()
	{
		// TODO: calc suitable position based on Engine.GetCanvasSize and/or other overly elemens ShockOverlay.GetOverlayRect
		int w, h, bm_w, bm_h;
		Engine.GetCanvasSize(w, h);
		ShockOverlay.GetBitmapSize(m_bgImage, bm_w, bm_h);
		m_rect.left = w-bm_w;
		m_rect.top =  0;
		m_rect.right = m_rect.left + bm_w;
		m_rect.bottom = m_rect.top + bm_h;
	}

	virtual void Draw()
	{
		ShockOverlay.DrawBitmap(m_bgImage, m_rect.left, m_rect.top);
		ShockOverlay.DrawString("Click me", m_rect.left+20, m_rect.top+12);
	}

	virtual BOOL MouseClick(int x, int y)
	{
		// do something
		ShockOverlay.ToggleLookCursor();

		return TRUE;
	}
};

// takes numeric keyboard input and draws string
class cHudElement_NumInput : public cHudElement
{
	enum { MAX_TEXT = 8 };
	char m_text[MAX_TEXT+2];
	int m_cursorpos;

public:
	cHudElement_NumInput()
	{
		m_text[0] = 0;
		m_cursorpos = 0;

		//ShockOverlay.SetCustomFont(0, "bignum", "fonts\\");

		CalcPlacement();
	}

	virtual ~cHudElement_NumInput() {}

	virtual void CalcPlacement()
	{
		// stick it somewhere in the middle
		int w, h;
		Engine.GetCanvasSize(w, h);
		m_rect.left = w/2;
		m_rect.top =  h/2 + 40;
		m_rect.right = m_rect.left + 100;
		m_rect.bottom = m_rect.top + 20;
	}

	virtual void OnShow(BOOL bShow)
	{
		m_text[0] = 0;
		m_cursorpos = 0;

		// make sure keypad HUD isn't active
		ShockGame.OverlayChange(kOverlayKeypad, kOverlayModeOff);

		ShockOverlay.SetKeyboardInputCapture(bShow);
	}

	virtual void Draw()
	{
		// draw a blinking underscore at cursor pos
		if ((ShockGame.SimTime()/500) & 1)
		{
			m_text[m_cursorpos] = '_';
			m_text[m_cursorpos+1] = 0;
		}

		ShockOverlay.SetTextColor(255, 255, 0);
		//ShockOverlay.SetFont(2);
		ShockOverlay.SetFont(1);
		ShockOverlay.DrawString(m_text, m_rect.left, m_rect.top);
		ShockOverlay.SetFont(0);
		ShockOverlay.SetTextColor(-1, 0, 0);

		m_text[m_cursorpos] = 0;
	}

	virtual BOOL OnKey(int ch, int keycode)
	{
		// process number key from main keyboard and numberic key pad both when numlock is on and off
		switch(keycode)
		{
		case '0':
		case '0'|KB_FLAG_2ND:
		case KEY_PAD_INS: ProcessKey(0); break;
		case '1':
		case '1'|KB_FLAG_2ND:
		case KEY_PAD_END: ProcessKey(1); break;
		case '2':
		case '2'|KB_FLAG_2ND:
		case KEY_PAD_DOWN: ProcessKey(2); break;
		case '3':
		case '3'|KB_FLAG_2ND:
		case KEY_PAD_PGDN: ProcessKey(3); break;
		case '4':
		case '4'|KB_FLAG_2ND:
		case KEY_PAD_LEFT: ProcessKey(4); break;
		case '5':
		case '5'|KB_FLAG_2ND:
		case KEY_PAD_CENTER: ProcessKey(5); break;
		case '6':
		case '6'|KB_FLAG_2ND:
		case KEY_PAD_RIGHT: ProcessKey(6); break;
		case '7':
		case '7'|KB_FLAG_2ND:
		case KEY_PAD_HOME: ProcessKey(7); break;
		case '8':
		case '8'|KB_FLAG_2ND:
		case KEY_PAD_UP: ProcessKey(8); break;
		case '9':
		case '9'|KB_FLAG_2ND:
		case KEY_PAD_PGUP: ProcessKey(9); break;

		case KEY_BS:
		case KEY_DEL:
		case KEY_PAD_DEL: ProcessKey(-1); break;

		case KEY_ENTER:
		case KEY_GREY_ENTER:
			// dactivate HUD element on return/enter
			Hide();
			break;

		default:
			// unhandled key
			return FALSE;
		}

		return TRUE;
	}

	void ProcessKey(int num)
	{
		if (num == -1)
		{
			// delete
			if (m_cursorpos > 0)
			{
				m_cursorpos--;
				m_text[m_cursorpos] = 0;
			}
			return;
		}

		if (m_cursorpos < MAX_TEXT)
		{
			m_text[m_cursorpos] = '0' + num;
			m_cursorpos++;
			m_text[m_cursorpos] = 0;
		}
	}
};

///////////////////////////////////////////////////////////////////////////////

// base class for our transparent (non-interactive) overlay elements
class cOverlayElement
{
public:
	BOOL m_bActive;
	int m_handle;

public:
	cOverlayElement() { m_bActive=FALSE; m_handle=-1; }
	virtual ~cOverlayElement() {}

	void Toggle() { m_bActive = !m_bActive; }
	void Show() { m_bActive = TRUE; }
	void Hide() { m_bActive = FALSE; }

	virtual void CalcPlacement() {}

	virtual void Draw() = 0;
};

// a custom overlay element
class cOverlayElement_Something : public cOverlayElement
{
protected:
	void CalcPos(int &x, int &y)
	{
		// place it where the alarm icon would normal be shown
		int w, h;
		ShockOverlay.GetOverlayRect(kOverlayAlarm, x, y, w, h);
	}

public:
	cOverlayElement_Something()
	{
		// create a simple static overlay of a bitmap

		int x, y;
		CalcPos(x, y);

		int bm = ShockOverlay.GetBitmap("hackicon", "iface\\");

		m_handle = ShockOverlay.CreateTOverlayItemFromBitmap(x, y, 127, bm, FALSE);
	}

	virtual ~cOverlayElement_Something() {}

	virtual void CalcPlacement()
	{
		int x, y;
		CalcPos(x, y);

		ShockOverlay.UpdateTOverlayPosition(m_handle, x, y);
	}

	virtual void Draw()
	{
		ShockOverlay.DrawTOverlayItem(m_handle);
	}
};

// another custom overlay element
class cOverlayElement_SomethingElse : public cOverlayElement
{
	BOOL m_bUpdateContents;

	int m_bgImage;

protected:
	void CalcPos(int &x, int &y)
	{
		// place it right of where the alarm icon would normal be shown
		int w, h;
		ShockOverlay.GetOverlayRect(kOverlayAlarm, x, y, w, h);
		x += w + 4;
	}

public:
	cOverlayElement_SomethingElse()
	{
		// create a dynamic/comlpex overlay with 64x64 size

		int x, y;
		CalcPos(x, y);

		m_handle = ShockOverlay.CreateTOverlayItem(x, y, 64, 64, 127, FALSE);
		m_bUpdateContents = TRUE;

		// get our bg bitmap
		m_bgImage = ShockOverlay.GetBitmap("alarm", "iface\\");
	}

	virtual ~cOverlayElement_SomethingElse() {}

	virtual void CalcPlacement()
	{
		int x, y;
		CalcPos(x, y);

		ShockOverlay.UpdateTOverlayPosition(m_handle, x, y);
	}

	virtual void Draw()
	{
		// draw overlay contents if something changed
		if (m_bUpdateContents)
		{
			m_bUpdateContents = FALSE;

			if ( ShockOverlay.BeginTOverlayUpdate(m_handle) )
			{
				char s[16];
				sprintf(s, "%d", ShockGame.GetPlayerPsiPoints());

				int w, h;
				ShockOverlay.GetStringSize(s, w, h);

				ShockOverlay.DrawBitmap(m_bgImage, 0, 0);
				ShockOverlay.DrawString(s, 64-w-4, 64-16);

				ShockOverlay.EndTOverlayUpdate();
			}
		}

		ShockOverlay.DrawTOverlayItem(m_handle);
	}
};


///////////////////////////////////////////////////////////////////////////////
// custom overlay handler

class cMyShockOverlay : public IShockOverlayHandler
{
	enum
	{
		NUM_ELEMS = 3,
		NUM_OVERLAYS = 2
	};

	cHudElement *m_elems[NUM_ELEMS];
	cOverlayElement *m_overlays[NUM_OVERLAYS];

public:
	cHudElement* FindElementFromPt(int x, int y)
	{
		for (int i=0; i<NUM_ELEMS; i++)
			if (m_elems[i]->m_bActive && m_elems[i]->m_rect.IsPtInside(x, y))
				return m_elems[i];

		return NULL;
	}

	void Init()
	{
		m_elems[0] = new cHudElement_Something;
		m_elems[1] = new cHudElement_SomethingElse;
		m_elems[2] = new cHudElement_NumInput;

		m_overlays[0] = new cOverlayElement_Something;
		m_overlays[1] = new cOverlayElement_SomethingElse;

		// show em all
		m_elems[0]->Show();
		m_elems[1]->Show();
		m_elems[2]->Show();
		m_overlays[0]->Show();
		m_overlays[1]->Show();
	}

	void Term()
	{
		for (int i=0; i<NUM_ELEMS; i++)
		{
			if (m_elems[i])
			{
				delete m_elems[i];
				m_elems[i] = NULL;
			}
		}
	}

public:
	//
	// IShockOverlayHandler interface
	//

	STDMETHOD_(void,DrawHUD)()
	{
		for (int i=0; i<NUM_ELEMS; i++)
			if (m_elems[i]->m_bActive)
				m_elems[i]->Draw();
	}

	STDMETHOD_(void,DrawTOverlay)()
	{
		for (int i=0; i<NUM_OVERLAYS; i++)
			if (m_overlays[i]->m_bActive)
				m_overlays[i]->Draw();
	}

	STDMETHOD_(void,OnUIEnterMode)()
	{
		int i;

		for (i=0; i<NUM_ELEMS; i++)
		{
			m_elems[i]->CalcPlacement();

			if (m_elems[i]->m_bActive)
				m_elems[i]->OnShow(TRUE);
		}

		for (i=0; i<NUM_OVERLAYS; i++)
			m_overlays[i]->CalcPlacement();
	}

	STDMETHOD_(BOOL,CanEnableElement)(int which)
	{
		// prevent the alarm icon from being displayed
		if (which == kOverlayAlarm)
			return FALSE;
		// deactivate numeric input HUD elem if keypad is about to be shown
		else if (which == kOverlayKeypad)
			m_elems[2]->Hide();

		return TRUE;
	}

	STDMETHOD_(BOOL,IsMouseOver)(int x, int y)
	{
		// see if mouse cursor is inside any of our custom elements
		return FindElementFromPt(x, y) != NULL;
	}

	STDMETHOD_(BOOL,MouseClick)(int x, int y)
	{
		cHudElement *pElem = FindElementFromPt(x, y);
		if (pElem)
			return pElem->MouseClick(x, y);

		return FALSE;
	}

	STDMETHOD_(BOOL,MouseDblClick)(int x, int y)
	{
		cHudElement *pElem = FindElementFromPt(x, y);
		if (pElem)
			return pElem->MouseDblClick(x, y);

		return FALSE;
	}

	STDMETHOD_(BOOL,MouseDragDrop)(int x, int y, BOOL start_drag, int cursor_mode)
	{
		cHudElement *pElem = FindElementFromPt(x, y);
		if (pElem)
			return pElem->MouseDragDrop(x, y, start_drag, cursor_mode);

		return FALSE;
	}

	STDMETHOD_(BOOL,HandleKey)(THIS_ int ch, int keycode)
	{
		if (keycode & KB_FLAG_DOWN)
		{
			int i;

			for (i=0; i<NUM_ELEMS; i++)
				if ( m_elems[i]->OnKey(ch, keycode & ~KB_FLAG_DOWN) )
					return TRUE;
		}

		return FALSE;
	}
};


cMyShockOverlay myOverlay;


///////////////////////////////////////////////////////////////////////////////
// base script that installs the custom overlay/HUD handler

BEGIN_SCRIPT(MyHudScript, RootScript)
METHODS:
MESSAGES:
	OnBeginScript()
	{
		ShockOverlay.SetHandler(&myOverlay);
		myOverlay.Init();

		DefaultOnBeginScript();
	}

	OnEndScript()
	{
		ShockOverlay.SetHandler(NULL);
		myOverlay.Term();

		DefaultOnEndScript();
	}
END_SCRIPT(MyHudScript)
